/*
	Spawn protection module
	Version 1.2
	By Boco

	Add to spawnPlayer() before giving health:
		self thread module\_mod_b_spawnprotect::main();
	Add to Callback_StartGameType() in the precaching section:
		module\_mod_b_spawnprotect::precache();
	Add to Callback_PlayerDamage() at the top:
		if ( self module\_mod_b_spawnprotect::checkForProtect() )
			return;

	This module uses the following cvars:
		b_spawn_protect_allies		- int, time in seconds to protect allied team.
		b_spawn_protect_axis		- int, time in seconds to protect axis team.
		b_spawn_protect_noShoot		- int, 0 or 1, turns off or on the disabling of spawn protection if you shoot.
*/

precache()
{
	// Boco: Set our invulnerable icon.
	game["headicon_invulnerable"] = "gfx/hud/hud@health_cross.tga";

	// Boco: Cache our invulnerable icon.
	precacheHeadIcon(game["headicon_invulnerable"]);
	precacheShader(game["headicon_invulnerable"]);
}

checkForProtect()
{
	if ( isDefined(self.invulnerable) && self.invulnerable == true )
		return true;
	else
		return false;
}

// Boco: Spawn invulnerability system.
main()
{
	if ( !isDefined(self.invulnerable) )
		self.invulnerable = false;

	if ( self.pers["team"] == "spectator" )
		return;
	if ( self.sessionstate == "spectator" )
		return;

	// Quit if player spawned (to call thread again).
	self endon("spawned");

	itimer = getcvarint( "b_spawn_protect_" + self.pers["team"] );
	if(!isdefined(self.invuln_icon))
	{
		self.invuln_icon = newClientHudElem(self);				
		self.invuln_icon.alignX = "left";
		self.invuln_icon.alignY = "top";
		self.invuln_icon.x = 602;
		self.invuln_icon.y = 377;
		self.invuln_icon.alpha = 0.4;
		self.invuln_icon.sort = 0;
		self.invuln_icon setShader(game["headicon_invulnerable"], 31, 23);
	}

	if(!isdefined(self.invuln_timer))
	{
		self.invuln_timer = newClientHudElem(self);
		self.invuln_timer.archived = false;
		self.invuln_timer.x = 607;	//+3 +5
		self.invuln_timer.y = 381;	//+4
		self.invuln_timer.alignX = "left";
		self.invuln_timer.alignY = "top";
		self.invuln_timer.fontScale = 1;
		self.invuln_timer.sort = 1;
		self.invuln_timer.color = (0, 1, 0);
	}
	self.invuln_timer setTimer(itimer);

	if(self.pers["team"] == "allies")
	{
		self.headicon = game["headicon_invulnerable"];
		self.headiconteam = "axis";
	} else {
		self.headicon = game["headicon_invulnerable"];
		self.headiconteam = "allies";
	}
	self.invulnerable = true;

	// Boco: Create the support threads.
	self thread spawnProtect_timer(itimer);
	self thread spawnProtect_spawn();
	if ( getcvarint("b_spawn_protect_noShoot") == 1 )
		self thread spawnProtect_noShoot();
	self waittill("invuln_timeout");
	self spawnProtect_cleanup();
}

spawnProtect_noShoot()
{
	self endon("invuln_timeout");
	for (;;)
	{
		if ( self.pers["team"] == "spectator" || self.sessionstate == "spectator" )
		{
			self.invulnerable = false;
			self notify("invuln_timeout");
		}

		if ( self attackButtonPressed() )
		{
			if(level.drawfriend)
			{
				if(self.pers["team"] == "allies")
				{
					self.headicon = game["headicon_allies"];
					self.headiconteam = "allies";
				} else {
					self.headicon = game["headicon_axis"];
					self.headiconteam = "axis";
				}
			} else {
				self.headicon = "";
				self.headiconteam = "";
			}
			self.invulnerable = false;
			self notify("invuln_timeout");
		}
		wait 0.1;
	}
}

spawnProtect_timer(time)
{
	self endon("invuln_timeout");
	wait( time );

	if ( self.pers["team"] != "spectator" && self.sessionstate != "spectator" )
	{
		if(level.drawfriend)
		{
			if(self.pers["team"] == "allies")
			{
				self.headicon = game["headicon_allies"];
				self.headiconteam = "allies";
			} else {
				self.headicon = game["headicon_axis"];
				self.headiconteam = "axis";
			}
		} else {
			self.headicon = "";
			self.headiconteam = "";
		}
	}

	self.invulnerable = false;
	self notify("invuln_timeout");
}

// Boco: Remove the queue HUD.
spawnProtect_cleanup()
{
	if ( isdefined(self.invuln_icon) )
		self.invuln_icon destroy();
	if ( isdefined(self.invuln_timer) )
		self.invuln_timer destroy();
}

// Boco: If the player spawns, close the queue HUD.
spawnProtect_spawn()
{
	self endon("invuln_timeout");
	self waittill("spawned");
	spawnProtect_cleanup();
}
